package org.one.stone.soup.file;

import java.io.*;
import java.util.*;

import org.one.stone.soup.util.*;

/*
 * Wet-Wired.com Library Version 2.1
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

 /**
	* The FileWatch class calls the fileChanged method of registered
	* listeners implementing the FileChangeListener when the selected
	* files modified time stamp changes.
	*
	* Instances of FileChangeListener can register to be informed when
	* a file changes by calling addListener.
	*
*/

public class FileWatch implements java.awt.event.ActionListener{

	private boolean processing = false;
	private TimeWatch timer;
	private long ts;
	private long size;
	private File file;
	private Vector listeners = new Vector();
	/**
	 * Creates an instance of FilesWatch to monitor changes to a file
	 * @param fileName		the name of the file to watch
	 * @param refreshTime	the time between rechecking the file last modified time in 1/1000 seconds
	 */

public FileWatch(String fileName,int refreshTime) {
	file = new File(fileName);

	if(!file.exists())
		return;

	ts = file.lastModified();
	size = file.length();

	timer = new TimeWatch(refreshTime,fileName);
	timer.setLooped(true);
	timer.addActionListener(this);
	timer.start();
}
/**
 * called by the timer to recheck the file modified time.
 */
public void actionPerformed(java.awt.event.ActionEvent e)
{
	if(processing)
	{
		return;
	}
	
	processing=true;
	if(file.lastModified()!=ts || file.length()!=size)
	{
		notifyListeners();
	}

	ts = file.lastModified();
	size = file.length();
	processing=false;
}
	/**
	 * Adds the specified file change listener to receive file change
	 * events from this component.
	 *
	 * @param    listener	the file change listener.
	 */

public void addListener(FileChangeListener listener)
{
	listeners.addElement(listener);
}
/**
 * Notifies the listeners that a file has changed
 */
private void notifyListeners()
{
	for(int loop=0;loop<listeners.size();loop++)
	{
		FileChangeListener listener = (FileChangeListener)listeners.elementAt(loop);
		listener.fileChanged(file);
	}
}
	/**
	 * Removes the specified file change listener from the list of
	 * registered listeners.
	 *
	 * @param    listener	the file change listener to remove.
	 */

public void removerListener(FileChangeListener listener)
{
	listeners.removeElement(listener);
}

public void removeAllListeners()
{
    listeners = new Vector();
}

public void stop()
{
    timer.stop();
    timer=null;
}

}
