package org.one.stone.soup.mjdb.data.field;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

/**
	* The Point2DField class is a wrapper for combined x and y int values.
	* @author Nik Cross
*/

public class Point2DField extends Field {
	public static String DEFAULT_ID = "_Point2D";

	private IntField x = new IntField(0);
	private IntField y = new IntField(0);
/**
 * This constructor creates a new Point2DField with an x value of 0 and
 * a y value of 0.
 */

public Point2DField() {
	super();
}
/**
 * This constructor sets the initial x and y values.
 * @param x		The x int value
 * @param y		The y int value
 */

public Point2DField(int x,int y) {
	super();

	try{
		this.x.setValue(x,this);
		this.y.setValue(y,this);
	}
	catch(LockException le)
	{}//Dummy catch
}
/**
 * This constructor sets the initial x and y values, and their ranges.
 * @param x			The x int value
 * @param xMin		The minimum x int value
 * @param xMax		The maximum x int value
 * @param y			The y int value
 * @param yMin		The minimum y int value
 * @param yMax		The maximum y int value
 */

public Point2DField(int x,int xMin,int xMax,int y,int yMin,int yMax) {
	super();

	this.x = new IntField(x,xMin,xMax);
	this.y = new IntField(y,yMin,yMax);

}
/**
 * Not implemented yet
 * @param target	The Field to compare with (Accepts Point2DField)
 */

public int compare(Field target) throws CompareException {

	throw new CompareException("Can't compare "+target+" not implemented");

}
/**
 * Returns a field held in this field as a String.
 * If name = "x" this method returns the  x int value
 * If name = "xMin" this method returns the minimum x value
 * If name = "xMax" this method returns the maximum x value
 * If name = "y" this method returns the  y int value
 * If name = "yMin" this method returns the minimum y value
 * If name = "yMax" this method returns the maximum y value
 * @param name	The name of the field to return
 */

public String getValue(String name) {
	if(name.equals("x"))
	{
		return ""+getValueX();
	}
	else if(name.equals("xMin"))
	{
		return ""+getValueXMin();
	}
	else if(name.equals("xMax"))
	{
		return ""+getValueXMax();
	}
	else if(name.equals("y"))
	{
		return ""+getValueY();
	}
	else if(name.equals("yMin"))
	{
		return ""+getValueYMin();
	}
	else if(name.equals("yMax"))
	{
		return ""+getValueYMax();
	}
	return "?";
}
/**
 * Returns the x int data held in this field.
 */

public int getValueX() {
	return x.getValue();
}
/**
 * Returns the maximum x value in this field.
 */

public int getValueXMax() {
	return x.getValueMax();
}
/**
 * Returns the minimum x value in this field.
 */

public int getValueXMin() {
	return x.getValueMin();
}
/**
 * Returns the y int data held in this field.
 */

public int getValueY() {
	return y.getValue();
}
/**
 * Returns the maximum y value in this field.
 */

public int getValueYMax() {
	return y.getValueMax();
}
/**
 * Returns the minimum y value in this field.
 */

public int getValueYMin() {
	return y.getValueMin();
}
/**
 * Returns true if either x or y has changed.
 */

public boolean hasChanged() {
	return x.hasChanged() | y.hasChanged();
}
/**
 * Resets the changed flag for this field to false.
 */

public void resetChanged()
{
	x.resetChanged();
	y.resetChanged();
	super.resetChanged();
}
/**
 * Set the minimum and maximum values for this field.
 * @param newXValueMin	The minimum x value
 * @param newXValueMax	The maximum x value
 * @param newYValueMin	The minimum y value
 * @param newYValueMax	The maximum y value
 */
public void setRange(int newXValueMin,int newYValueMin,int newXValueMax,int newYValueMax) {

	x.setValueMin(newXValueMin);
	x.setValueMax(newXValueMax);
	y.setValueMin(newYValueMin);
	y.setValueMax(newYValueMax);
}
/**
 * Set the int value.
 * If name = "x" then the x value is set
 * If name = "xMin" then the minimum x value is set
 * If name = "xMax" then the maximum x value is set
 * If name = "y" then the y value is set
 * If name = "yMin" then the minimum y value is set
 * If name = "yMax" then the maximum y value is set
 * @param name		The target field name
 * @param value		A String containing the new int value
 * @param source	The object requesting the value change
 */

public void setValue(String name, String value, Object source) throws LockException
{
	if(name.equals("x"))
	{
		setValueX( Integer.parseInt(value),this );
	}
	else if(name.equals("y"))
	{
		setValueY( Integer.parseInt(value),this );
	}
	else if(name.equals("xMin"))
	{
		setValueXMin( Integer.parseInt(value) );
	}
	else if(name.equals("xMax"))
	{
		setValueXMax( Integer.parseInt(value) );
	}
	else if(name.equals("yMin"))
	{
		setValueYMin( Integer.parseInt(value) );
	}
	else if(name.equals("yMax"))
	{
		setValueYMax( Integer.parseInt(value) );
	}
}
/**
 * Set the x value.
 * @param newData		The new x value
 * @param source		The object requesting the value change
 */

public void setValueX(int newData,Object source) throws LockException {

	if(x.getValue()==newData)
		return;

	x.setValue(newData,this);
	x.setChanged();
	setChanged();

	if(report)
		reportChange(source);
}
/**
 * Set the maximum x value.
 * @param newXValueMax		The new maximum x value
 * @param source		The object requesting the value change
 */

public void setValueXMax(int newXValueMax) {
	x.setValueMax(newXValueMax);
}
/**
 * Set the maximum y value.
 * @param newYValueMin		The new minimum y value
 * @param source		The object requesting the value change
 */

public void setValueXMin(int newXValueMin) {
	x.setValueMin(newXValueMin);
}
/**
 * Set the y value.
 * @param newData		The new y value
 * @param source		The object requesting the value change
 */

public void setValueY(int newData,Object source) throws LockException {

	if(y.getValue()==newData)
		return;

	y.setValue(newData,this);
	y.setChanged();
	setChanged();

	if(report)
		reportChange(source);
}
/**
 * Set the maximum y value.
 * @param newYValueMax		The new maximum y value
 * @param source		The object requesting the value change
 */

public void setValueYMax(int newYValueMax) {
	y.setValueMax(newYValueMax);
}
/**
 * Set the maximum y value.
 * @param newYValueMax		The new maximum y value
 * @param source		The object requesting the value change
 */

public void setValueYMin(int newYValueMin) {
	y.setValueMin(newYValueMin);
}
public String toString() {
	return ""+x.toString()+","+y.toString();
}
}
