package org.one.stone.soup.swing;

import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.Vector;

/*
 * Wet-Wired.com Library Version 2.1
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

/**
 * @author nikcross
 *
 */

// TODO
public class JPrintStreamConsole extends JScrollPane {

	private int maxChars=10000;
	private int characters = 0;

	private PrintStream out;
	private PrintWriter log;
	private Vector listeners;

	private JTextArea textArea;
	private OutputStreamPrint oStreamPrint;

	class OutputStreamPrint extends OutputStream{

		private boolean isBuffered = true;
		public OutputStreamPrint()
		{
		}

		public void setBuffered(boolean state)
		{
			this.isBuffered = state;
		}

		StringBuffer buffer = new StringBuffer();
		public void write(int data)
		{
			if(isBuffered)
			{
				buffer.append( (char)data );
				if(data==0x0D || data==0x0A)
				{
					printOut( buffer.toString() );
					buffer = new StringBuffer();
				}
			}
			else
			{
				printOut( ""+(char)data );
			}
		}
		
		private void writeToListeners(String data)
		{
		    for(int loop=0;loop<listeners.size();loop++)
		    {
		        OutputStream stream = (OutputStream)listeners.elementAt(loop);
		        try{
		            stream.write(data.getBytes());
		            stream.flush();
		        }
		        catch(Exception e)
		        {
		            listeners.remove(stream);
		            loop--;
		        }
		    }
		}
	}
public JPrintStreamConsole()
{
	this(20,80);
}
public JPrintStreamConsole(int rows, int columns) {

	textArea = new JTextArea(rows, columns);
	textArea.setEditable(false);

	setViewportView( textArea );

	oStreamPrint = new OutputStreamPrint();
	out = new PrintStream( oStreamPrint );
}

public void setRows(int rows)
{
	textArea.setRows(rows);
}

public void setColumns(int columns)
{
	textArea.setColumns(columns);
}

public void addOutputListener(OutputStream stream)
{
    listeners.add(stream);
}
public void removeOutputListener(OutputStream stream)
{
    listeners.remove(stream);
}
/**
 *
 */
public void clear()
{
	textArea.setText("");
	characters = 0;
}
/**
 * Insert the method's description here.
 * Creation date: (03/05/03 12:22:56)
 * @return int
 */
public int getMaxChars() {
	return maxChars;
}
public PrintStream getOut() {
	return out;
}
public void printOut(String data)
{
	if(log!=null)
	{
		log.print(data);
		log.flush();
	}

	textArea.append(data);
	characters+=data.length();
	
	textArea.setCaretPosition(characters);

	if(characters>maxChars)
	{
		String txt = textArea.getText();
		textArea.setText( txt.substring(txt.length()-1000) );
		characters = textArea.getText().length();
		
	}
}
public void setBackground(Color c)
{
	super.setBackground(c);

	if(textArea!=null)
	{
		textArea.setBackground(c);
	}
}
/**
 * Insert the method's description here.
 * Creation date: (25/08/03 10:22:28)
 * @param state boolean
 */
public void setBuffered(boolean state)
{
	oStreamPrint.setBuffered(state);
}
public void setForeground(Color c)
{
	super.setForeground(c);

	if(textArea!=null)
	{
		textArea.setForeground(c);
	}
}
public void setLogFile(String file)
{
	System.out.println("Now writing to log file '"+new File(file).getAbsolutePath()+"'");
	try{
		new File(file).getParentFile().mkdirs();
		log = new PrintWriter( new FileOutputStream(file) );
	}
	catch(IOException ie)
	{
		ie.printStackTrace();
	}
}
/**
 * Insert the method's description here.
 * Creation date: (03/05/03 12:22:56)
 * @param newMaxChars int
 */
public void setMaxChars(int newMaxChars) {
	maxChars = newMaxChars;
}
/**
 * Insert the method's description here.
 * Creation date: (24/08/03 16:16:14)
 * @param text java.lang.String
 */
public void setText(String text)
{
	textArea.setText(text);
}
}
