package org.one.stone.soup.xapp.components.spreadsheet;

import java.text.DecimalFormat;

import org.one.stone.soup.cursor.TableCursor;
import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.xml.XmlElement;

public class XappSpreadSheetAPI {

	private XappSpreadsheetModel spreadsheet;
	public TableCursor cursor = new TableCursor(-1,-1);
	
	public XappSpreadSheetAPI(XappSpreadsheetModel spreadsheet)
	{
		this.spreadsheet = spreadsheet;
	}
	
	public String getField(String cellName)
	{
		if( spreadsheet.getTableCursor(cellName).isValid()==false )
		{
			int column = spreadsheet.getColumnIndex(cellName);
			if(column==-1)
			{
				return null;
			}
			else
			{
				return spreadsheet.getField( cursor.row,column ).getValue();
			}
		}
		else
		{
			return spreadsheet.getValueAt(cellName);
		}
	}

	public TableCursor find(String data)
	{
		for(int loopC=0;loopC<spreadsheet.getColumnCount()-1;loopC++)
		{
			for(int loopR=0;loopR<spreadsheet.getRowCount();loopR++)
			{
				if( spreadsheet.getField(loopR,loopC).getValue().equals(data) )
				{
					return new TableCursor(loopC,loopR);
				}
			}
		}
		return null;
	}
	
	public void setField(String cellName,String value)
	{
		if( spreadsheet.getTableCursor(cellName).isValid()==false )
		{
			int column = spreadsheet.getColumnIndex(cellName);
			if(column!=-1)
			{
				spreadsheet.setField( cursor.row,column,value );
			}
		}
		else
		{
			 spreadsheet.setField(cellName,value);
		}
	}
	
	public void incField(String cellName)
	{
		int value = Integer.parseInt( getField(cellName) );
		value++;
		setField(cellName,""+value);		
	}

	public void decField(String cellName)
	{
		int value = Integer.parseInt( getField(cellName) );
		value++;
		setField(cellName,""+value);		
	}
	
	public void addRow()
	{
		XmlElement xData = spreadsheet.getData();
		int columns = xData.getElementByName("columns").getElementCount();
		XmlElement xRow = xData.addChild("row");
		for(int loop=0;loop<columns;loop++)
		{
			xRow.addChild("field").addValue("");
		}
		
		spreadsheet.getParent().setData(xData);
		spreadsheet = spreadsheet.getParent().getSpreadsheetModel();
	}	
	
	public int getNumberOfColumns()
	{
		return spreadsheet.getColumnCount();
	}
	
	public int getNumberOfRows()
	{
		return spreadsheet.getRowCount();
	}
	
	public String getColumnTitle(int column)
	{
		return spreadsheet.getColumnName(column);
	}

	public String format(Object value,String format)
	{
		if(format.equals("currency"))
		{
			return DecimalFormat.getCurrencyInstance().format(value);			
		}
		return DecimalFormat.getNumberInstance().format(value);
	}

	public float sum(String sum)
	{
		float result = 0;
		
		String[] cell = StringArrayHelper.parseFields(sum,',');
		
		for(int loop=0;loop<cell.length;loop++)
		{
			String rawValue = null;
			if(cell[loop].indexOf("..")!=-1)
			{
				result+= sumBlock( cell[loop] );
				continue;
			}
			else if( spreadsheet.getTableCursor(cell[loop]).isValid()==false )
			{
				spreadsheet.getField( cursor.row,spreadsheet.getColumnIndex(cell[loop]) );
			}
			else
			{
				rawValue = spreadsheet.getValueAt(cell[loop]);
			}
			float value = 0;
			if(rawValue!=null && rawValue.length()>0)
			{
				value = Float.parseFloat( rawValue );
			}
			
			result+=value;
		}
		
		return result;
	}
	
	private float sumBlock(String block)
	{
		String cellA = block.substring(0,block.indexOf(".."));
		String cellB = block.substring(block.indexOf("..")+2);
		
		TableCursor cursorA = spreadsheet.getTableCursor(cellA);
		TableCursor cursorB = spreadsheet.getTableCursor(cellB);
		
		if(cursorA.row>cursorB.row)
		{
			int hold = cursorA.row;
			cursorA.row = cursorB.row;
			cursorB.row = hold;
		}
		
		if(cursorA.field>cursorB.field)
		{
			int hold = cursorA.field;
			cursorA.field = cursorB.field;
			cursorB.field = hold;
		}		
		cursorB.row -= cursorA.row;
		cursorB.field -= cursorA.field;
		cursorB.row++;
		cursorB.field++;
		
		float result = 0;
		for(int loopR=0;loopR<cursorB.row;loopR++)
		{
			for(int loopC=0;loopC<cursorB.field;loopC++)
			{
				result+= Float.parseFloat( spreadsheet.getField(cursorA.row+loopR,cursorA.field+loopC).getValue() );
			}
		}
		return result;
	}
}
